var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { nameof } from "ts-simple-nameof";
import { Api } from "../../Api";
import { ExalusConnectionService } from "../ExalusConnectionService";
import { ConnectionState } from "../IExalusConnectionService";
import { TypedEvent } from "../../TypedEvent";
import { AccessLevel, Gender } from '../Users/IUser';
import { LoginError } from "./ISessionService";
import { DataFrame, Method, Status } from '../../DataFrame';
import { DependencyContainer } from "../../DependencyContainer";
import { User } from "../Users/User";
export class SessionService {
    constructor() {
        this._onUserLoggedInEvent = new TypedEvent();
        this._onUserLoggedOutEvent = new TypedEvent();
        this._serviceName = nameof(SessionService);
        this._password = "";
        this._user = null;
    }
    OnUserLoggedInEvent() {
        return this._onUserLoggedInEvent;
    }
    OnUserLoggedOutEvent() {
        return this._onUserLoggedOutEvent;
    }
    get User() {
        return this._user;
    }
    UserLogOutAsync() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            var logoutRequest = new DataFrame();
            logoutRequest.Resource = "/users/user/logout";
            logoutRequest.Method = Method.Put;
            var connection = Api.Get(ExalusConnectionService);
            connection.OnConnectionStateChangedEvent().Unsubscribe(this.OnReconnected);
            const result = yield Api.Get(ExalusConnectionService)
                .SendAndWaitForResponseAsync(logoutRequest, 20000, false);
            if (result.Status == Status.OK) {
                this._onUserLoggedOutEvent.Invoke(this._user);
                (_a = DependencyContainer.Log) === null || _a === void 0 ? void 0 : _a.Debug("User has been logged out.");
            }
            else
                (_b = DependencyContainer.Log) === null || _b === void 0 ? void 0 : _b.Debug("Failed to logout user!");
        });
    }
    OnReconnected(connectionState) {
        return __awaiter(this, void 0, void 0, function* () {
            if (connectionState == ConnectionState.Connected && this._user != null) {
                var connection = Api.Get(ExalusConnectionService);
                const result = yield connection.SendAndWaitForResponseAsync(new LoginUserRequest(this._user.Email, this._password), 20000, false);
                if (result.Status == Status.OK && result.Data != null) {
                    var user = new User();
                    this._user = user;
                    user.AccessLevel = result.Data.AccessLevel;
                    user.Email = result.Data.Email;
                    user.Gender = result.Data.Gender;
                    user.Guid = result.Data.Guid;
                    user.IsAccountOnline = result.Data.IsAccountOnline;
                    user.Name = result.Data.Name;
                    user.IsActive = result.Data.IsActive;
                    user.IsBanned = result.Data.IsBanned;
                    user.Phone = result.Data.Phone;
                    user.Surname = result.Data.Surname;
                    this._onUserLoggedInEvent.Invoke(this._user);
                }
            }
        });
    }
    UserLogInAsync(email, password) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            var connection = Api.Get(ExalusConnectionService);
            const result = yield connection.SendAndWaitForResponseAsync(new LoginUserRequest(email, password), 20000, false);
            connection.OnConnectionStateChangedEvent().Subscribe(this.OnReconnected);
            (_a = DependencyContainer.Log) === null || _a === void 0 ? void 0 : _a.Debug(`Did logging in succeded?: ${result.Status == Status.OK}`);
            if (result.Status == Status.OK && result.Data != null) {
                this._password = password;
                var user = new User();
                this._user = user;
                user.AccessLevel = result.Data.AccessLevel;
                user.Email = result.Data.Email;
                user.Gender = result.Data.Gender;
                user.Guid = result.Data.Guid;
                user.IsAccountOnline = result.Data.IsAccountOnline;
                user.Name = result.Data.Name;
                user.IsActive = result.Data.IsActive;
                user.IsBanned = result.Data.IsBanned;
                user.Phone = result.Data.Phone;
                user.Surname = result.Data.Surname;
                this._onUserLoggedInEvent.Invoke(this._user);
                return user;
            }
            else {
                (_b = DependencyContainer.Log) === null || _b === void 0 ? void 0 : _b.Debug(`User login response: ${result.Data}`);
                return LoginError.WrongAuthData;
            }
        });
    }
    GetServiceName() {
        return this._serviceName;
    }
}
export class UserData {
    constructor() {
        this.Guid = "";
        this.Name = "";
        this.Surname = "";
        this.Password = "";
        this.Email = "";
        this.Phone = "";
        this.IsActive = true;
        this.IsBanned = false;
        this.IsAccountOnline = false;
        this.AccessLevel = AccessLevel.Any;
        this.Gender = Gender.Unknown;
        this.ConfigurationTime = "";
        this.IsGeolocationSet = false;
        this.UpdatesChannel = "";
    }
}
class UserLoginData {
    constructor(email, password) {
        this.Email = "";
        this.Password = "";
        this.Email = email;
        this.Password = password;
    }
}
class LoginUserRequest extends DataFrame {
    constructor(email, password) {
        super();
        this.Data = new UserLoginData(email, password);
        this.Resource = "/users/user/login";
        this.Method = Method.Put;
    }
}
//# sourceMappingURL=SessionService.js.map